{-# OPTIONS -cpp #-}
module Utility where

import Control.Monad

-- | Monadic version of @if@.
mif :: Monad m => m Bool -> m a -> m a -> m a
mif g t e = g >>= \res -> if res then t else e

-- | Monadic version of @if@, lifting the branches.
mif' :: Monad m => m Bool -> a -> a -> m a
mif' g t e = g >>= \res -> if res then return t else return e

-- | Monadic version of '||'.
mOr :: Monad m => m Bool -> m Bool -> m Bool
mOr x y = mif x (return True) y

-- | Monadic version of '&&'.
mAnd :: Monad m => m Bool -> m Bool -> m Bool
mAnd x y = mif x y (return False)

-- | Monadic version of 'any'.
anyM :: Monad m => (a -> m Bool) -> [a] -> m Bool
anyM f = foldr (mOr . f) (return False)

-- | Monadic version of 'all'.
allM :: Monad m => (a -> m Bool) -> [a] -> m Bool
allM f = foldr (mAnd . f) (return True)

-- | Monadic version of 'or'.
mor :: Monad m => [m Bool] -> m Bool
mor  = anyM id

-- | Monadic version of 'and'.
mand :: Monad m => [m Bool] -> m Bool
mand = allM id


-- This allows for specialization of mor and mand.

{-# INLINE mif #-}
{-# INLINE mif' #-}
{-# INLINE mor #-}
{-# INLINE mand #-}
